/*
 * @copyright		© Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module			ep.ui.dialog
 *
 * @revision		$Revision: 1.15.16.3 $
 */

jQuery.plugin( 'ep.ui.dialog', [ 'ep.core', 'ep.ui.form', 'ep.ui.input', 'jQuery.ui.dialog'], function( $, undefined ){

	$.widget('ep.ui.Dialog',$.ui.dialog,{

		options: {
			resizable:	'se',
			zIndex:		20000
		},

		_create: function(){
			this.Inherited(arguments);

			var self = this,
				o = self.options;

			if( o.form ){
				self._createForm(o.form);
			}

			self.uiDialog
				.removeClass('ui-corner-all')
				.addClass('epDialog');
			self.uiDialogTitlebar
				.removeClass('ui-corner-all')
				.addClass('epDialogTitleBar');

			self._setOption( "closeText", o.closeText );
		},

		_setOption: function( key, value ){
			var self = this;

			switch (key) {
				case 'form':
					if( !self.uiForm ){
						self._createForm(value);
					}
					else{
						self.uiForm.uiForm(value);
					}
				break;
				case 'title':
					$(".ui-dialog-title", self.uiDialogTitlebar)
						.empty()
						.append( self.options.title = (value || '&#160;') );
				break;
				case 'closeText':
					self.uiDialogTitlebar.find(".ui-dialog-titlebar-close").toggle(value !== false);
					value = value === true ? "close" : (value || "");
				default:
					self.Inherited(arguments);
				break;
			}
		},

		_createForm: function( form ){
			this.uiForm = this.options.form = ep('<form>')
				.attr(this.options.form)
				.insertBefore(this.uiDialog)
				.append(this.uiDialog)
				.uiForm();
		},

		_createButtons: function( buttons ) {
			var self = this,
				hasButtons = false,
				uiDialogButtonPane = $('<div>')
					.addClass(
						'ui-dialog-buttonpane ' +
						'ui-widget-content ' +
						'ui-helper-clearfix ' +
						'epDialogButtonBar'
					),
				uiButtonSet = $( "<div>" )
					.addClass( "ui-dialog-buttonset" )
					.appendTo( uiDialogButtonPane );

			// if we already have a button pane, remove it
			self.uiDialog.find('.ui-dialog-buttonpane').remove();

			if( typeof buttons === 'object' && buttons !== null ){
				$.each(buttons, function() {
					return !(hasButtons = true);
				});
			}
			if( hasButtons ){
				$.each(buttons, function(name, props) {
					props = $.extend({
						'type':		'button',
						'text':		name,
						'click':	function(){}
					}, $.isFunction( props ) ? { click: props, text: name } : props );

					// type cann't set with .attr()
					var type = props.type;
					delete(props.type);

					buttons[name] = ep('<button type="'+type+'">')
						.attr( props, true )
						.off('click')
						.appendTo(uiButtonSet)
						.uiInput()
						.on( 'click', function(){
							props.click.apply(self.element[0], arguments);
						});
				});
				uiDialogButtonPane.appendTo(self.uiDialog);
			}
		},

		open: function(){
			this.options.modal && $('body').css('overflow', 'hidden');
			this.Inherited(arguments);
		},

		close: function(){
			this.options.modal && $('body').css('overflow', 'auto');
			this.Inherited(arguments);
		},

		destroy: function(){
			this.Inherited(arguments);

			if( this.uiForm ){
				this.uiForm.remove();
			}
		}

	});

 var overlayCreate = $.ui.dialog.overlay.create;

 $.ui.dialog.overlay.create = function( dialog ){
  var length = this.instances.length,
   ret = overlayCreate.apply( this, arguments );

  if( length === 0 ){
   // allow closing by click the overlay
   $(document).on('click.dialog-overlay', function(event) {
    if( dialog.options.closeOnClickOut && event.originalTarget == dialog.overlay.$el[0] ){
     dialog.close(event);
     event.preventDefault();
    }
   });
  }
  return ret;
 };

});