use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    LoadObjectByPath
    LoadRootObject
);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::UPS::API::DataCollectors;
use DE_EPAGES::UPS::API::XML::Builder::PaymentInformationBuilder;
use Readonly;
use Test::More;
use Test::Exception;

use Data::Dumper;
Readonly my $ROOTPATH => '/TestStore';
## no critic (ProhibitLongLines)
sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # startUp
            ok( Init(), 'initialization' );

            # tests
            eval {
                my $ShippingLabelUPS = LoadObjectByPath($ROOTPATH.'/TestShop/Customers/1001/Orders/1002/ShippingLabels/1001');## no critic (ProhibitPathConcatenation)
                isa_ok($ShippingLabelUPS, 'DE_EPAGES::UPS::API::Object::ShippingLabelUPS');
                my $DataCollectors = DE_EPAGES::UPS::API::DataCollectors->new($ShippingLabelUPS);
                isa_ok($DataCollectors, 'DE_EPAGES::UPS::API::DataCollectors');
                my $hShippingMethod = $DataCollectors->getShippingFulFillmentMethodData();
                my $paymentInformationBuilder = DE_EPAGES::UPS::API::XML::Builder::PaymentInformationBuilder->new($hShippingMethod);
                testType($paymentInformationBuilder);
                testBillshipper($paymentInformationBuilder);
                testShipmentCharge($paymentInformationBuilder);
                testAssemble($paymentInformationBuilder);

            };

            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }

            # shutDown
            ok( Cleanup(), 'cleanup' );
            done_testing();
        }
    );
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'TestStore' } );
    ImportObjects( 'importOrder.xml', $ROOTPATH );
    return 1;
}
## no critic (ProhibitPrivateSubroutineUse)
sub testType {
    my ($paymentInformationBuilder) = @_;
    is ($paymentInformationBuilder->_type()->{'Type'},'01','call Type() / Type');
}

sub testBillshipper {
    my ($paymentInformationBuilder) = @_;
    is ($paymentInformationBuilder->_billShipper()->{'BillShipper'}->{'AccountNumber'},'ShipperNumber','call _BillShipper() BillShipper/AccountNumber');
}

sub testShipmentCharge {
    my ($paymentInformationBuilder) = @_;
    my $ShipmentCharge = $paymentInformationBuilder->_shipmentCharge();
    is ($ShipmentCharge->{'ShipmentCharge'}->{'BillShipper'}->{'AccountNumber'},'ShipperNumber','call _ShipmentCharge() ShipmentCharge/AccountNumber');
    is ($ShipmentCharge->{'ShipmentCharge'}->{'Type'},'01','call _ShipmentCharge() ShipmentCharge/Type');
}
sub testAssemble {
    my ($paymentInformationBuilder) = @_;
    my $assemble = $paymentInformationBuilder->assemble();
    is ($assemble->{'PaymentInformation'}->{'ShipmentCharge'}->{'BillShipper'}->{'AccountNumber'},'ShipperNumber','call assemble() PaymentInformation/ShipmentCharge/AccountNumber');
    is ($assemble->{'PaymentInformation'}->{'ShipmentCharge'}->{'Type'},'01','call assemble() PaymentInformation/ShipmentCharge/Type}');
}

RunScript( Sub => \&Main );

1;